package at.ac.uibk.gitsearch.security.oauth2;

import java.util.Base64;
import java.util.List;
import java.util.Optional;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.gitlab4j.api.Constants.Encoding;
import org.gitlab4j.api.Constants.SearchScope;
import org.gitlab4j.api.GitLabApi;
import org.gitlab4j.api.GitLabApiException;
import org.gitlab4j.api.models.Namespace;
import org.gitlab4j.api.models.Project;
import org.gitlab4j.api.models.RepositoryFile;

public class TestMetaDataGenerator {

	private final static Logger logger = LogManager.getLogger(TestMetaDataGenerator.class);

	private static final String metaDataContent = "metadataVersion: \"0.2\"\r\n" + "type: collection\r\n"
			+ "collectionContent: \r\n" + "  - src/if-001/if-001.yml\r\n" + "  - src/if-002/if-002.yml\r\n"
			+ "  - src/if-003/if-003.yml\r\n" + "identifier: javaCourseTUWienTest\r\n"
			+ "structure: hierarchical # one from atomic, networked, hierarchical, linear\r\n"
			+ "version: \"1.0\" # just a version tag\r\n"
			+ "status: final # one fo draft, final, revised, unavalable\r\n" + "title: ${title}\r\n"
			+ "description: \"Dies sind Teile des Einführungskurses an der TU Wien. \r\n"
			+ "   Momentan hier genutzt zum Testen der Metadateninfrastruktur. Version 1.xxx\r\n" + "   3. Zeile\"\r\n"
			+ "programmingLanguage: \r\n" + "  - JAVA\r\n" + "language: [de]\r\n"
			+ "educationLevel: \"Anfänger, (to be detailed)\"\r\n" + "audience: \"Anfaenger\"\r\n"
			+ "keyword: [Java, IOTest, latex, ${keyword}]\r\n" + "license: MIT # mandatory\r\n" + "creator: \r\n"
			+ "  - {name: \"Stefan Podlipnig\", affiliation: \"TU Wien\", email: \"stefan.podlipnig@tuwien.ac.at\"}\r\n"
			+ "contributor:\r\n"
			+ "  - {name: \"Daniel Bastta\", affiliation: \"TU Wien\", email: \"daniel.bastta@tuwien.ac.at\"}\r\n"
			+ "  - {name: \"Andreas Merckel\", affiliation: \"TU Wien\", email: \"andreas.merkel@tuwien.ac.at\"}\r\n"
			+ "  - {name: \"Kerstin Limbeck\", affiliation: \"TU Wien\", email: \"kerstin.limbeck@tuwien.ac.at\"}\r\n"
			+ "publisher:\r\n"
			+ "  - {name: \"Andreas Merckel\", affiliation: \"TU Wien\", email: \"andreas.merkel@tuwien.ac.at\"}\r\n"
			+ "\r\n" + "format: [latex]\r\n" + "deprecated: false\r\n" + "difficulty: simple\r\n" + "valid:\r\n"
			+ "  start: 2020-01-01\r\n" + "  end: 2030-12-31\r\n" + "requires: [Java14]\r\n"
			+ "image: ./courseIcon.png\r\n" + "";

	public String getTestMetaData(int count) {
		return metaDataContent.replace("${title}", "Meta Test Data " + count).replace("${keyword}", "testing" + count);
	}

	/**
	 * just a weird location to generate Test Data :-(. Should be purged as soon as
	 * possible .
	 * 
	 * @throws GitLabApiException
	 */
	public void generateTestData(GitLabApi gitLabApi, String issuerURI, String idToken) throws GitLabApiException {
		final String projectPrefix = "apiTest";
		gitLabApi.enableRequestResponseLogging();
		boolean skipDelete = true;
		if (!skipDelete) {
			final List<Project> projects = (List<Project>) gitLabApi.getSearchApi().globalSearch(SearchScope.PROJECTS,
					projectPrefix);
			projects.stream().forEach(p -> {
				try {
					gitLabApi.getProjectApi().deleteProject(p.getId());
				} catch (GitLabApiException e) {
					logger.warn("Cannot delete project {}", p.getName(), e);
				}
			});
		}

		final List<Namespace> namespaces = gitLabApi.getNamespaceApi().getNamespaces();
		Optional<Namespace> heathCheckNSO = namespaces.stream().filter(ns -> ns.getName().equals("health-check-tests"))
				.findFirst();
		if (heathCheckNSO.isEmpty()) {
			logger.warn("Namespace health-check-tests not found");
			return;
		}
		Namespace healthNS = heathCheckNSO.get();

		for (int i = 1; i <= 100; i++) {
			Project p = new Project();
			p.setName("apiTest_" + i + "_" + System.currentTimeMillis());
			p.setDescription("Automatically generated testapi");
			p.setNamespace(healthNS);
			p.setSharedWithGroups(null);
			
		
			logger.info("Creating project {} in {}", p.getName(), healthNS.getFullPath());

			Project createdProject = gitLabApi.getProjectApi().createProject(p);

			gitLabApi.getProjectApi();
			RepositoryFile rf = new RepositoryFile();
			rf.setFileName("metadata.yml");
			String base64encoded = Base64.getEncoder().encodeToString(getTestMetaData(i).getBytes());
			rf.setContent(base64encoded);
			rf.setEncoding(Encoding.BASE64);
			rf.setFilePath("metadata.yml");

			gitLabApi.getRepositoryFileApi().createFile(createdProject.getId(), rf, "master",
					"automagically generated metadata :-)");
		}

	}

}
