package at.ac.uibk.gitsearch.security.oauth2;

import java.time.Instant;
import java.util.function.Consumer;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.gitlab4j.api.Constants.TokenType;
import org.gitlab4j.api.GitLabApi;
import org.gitlab4j.api.GitLabApiException;
import org.gitlab4j.api.models.User;
import org.springframework.security.oauth2.client.authentication.OAuth2LoginAuthenticationToken;
import org.springframework.security.oauth2.client.registration.ClientRegistration.ProviderDetails;
import org.springframework.security.oauth2.core.oidc.user.OidcUser;
import org.springframework.stereotype.Component;

import at.ac.uibk.gitsearch.service.dto.UserDTO;

@Component
public class UserDetailsFetcher {
	
	private final static Logger logger = LogManager.getLogger(UserDetailsFetcher.class);
	
	private final static TestMetaDataGenerator testMetaDataGenerator = new TestMetaDataGenerator();


	
	/** fills in data from OIDC repository.
	 *   returns true, if user data really changed **/
	public boolean updateUserDetails(UserDTO u, OidcUser oidcUser, OAuth2LoginAuthenticationToken accessToken) {
			String idToken = accessToken.getAccessToken().getTokenValue();
			ProviderDetails providerDetails = accessToken.getClientRegistration().getProviderDetails();
			String issuerURI = (String) providerDetails.getConfigurationMetadata().get("issuer");
        try( GitLabApi gitLabApi = new GitLabApi(issuerURI, TokenType.OAUTH2_ACCESS, idToken)) {
        	boolean modified = false;
	        gitLabApi.enableRequestResponseLogging();
	        // List<Project> memberProjects = gitLabApi.getProjectApi().getMemberProjects();
	        User gitUser = gitLabApi.getUserApi().getCurrentUser();
	        modified |= updateAttribute(gitUser.getAvatarUrl(),u.getImageUrl(), u::setImageUrl);
	        modified |= updateAttribute(gitUser.getEmail(), u.getEmail(), u::setEmail);
	        modified |= updateAttribute(gitUser.getName(),u.getLastName(), u::setLastName);
	        // modified |= updateAttribute(gitUser.getUsername(),u.getLogin(), u::setLogin);
	        
	        
			if (modified) u.setLastModifiedDate(Instant.now());
			// testMetaDataGenerator.generateTestData(gitLabApi, issuerURI, idToken);
			return modified;
        } catch (GitLabApiException e) {
			logger.warn("Cannot fetch details for user {}", oidcUser.getName(), e);
			
		}
        return false;
	}
	
	private static boolean updateAttribute (String newString, String oldString, Consumer<String> setter) {
		if(oldString==null) {
			if(newString == null) return false;
		} else if(oldString.equals(newString)) {
			return false;
		}
		setter.accept(newString);
		return true;
	}
	
}
